﻿/*
 * author:Joey Zhao
 * date:2010-10-5 
 * describe:edit form
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TodoList.PublicUnit;
using TodoList.Db;
using System.Configuration;

namespace TodoList
{
    public partial class FormEdit : Form
    {
        private T_CONTENT _content;
        private PublicUnit.PublicEnums.EditType _eidtType;
        private readonly string CON_ALL = "全部";
        private int _catalogId;//目录ID

        public FormEdit()
        {
            InitializeComponent();
            this.checkBoxExe.CheckedChanged += delegate(object o, EventArgs e) 
            {
                this.buttonExe.Enabled = this.checkBoxExe.Checked ? true : false;
                this.buttonClean.Enabled = this.checkBoxExe.Checked ? true : false;
            };
            this.buttonClean.Click += delegate(object o, EventArgs e) { this.textBoxExePath.Text = string.Empty; };
            this.buttonExe.Click += new EventHandler(buttonExe_Click);
            this.buttonCancel.Click += delegate(object o, EventArgs e) { this.DialogResult = System.Windows.Forms.DialogResult.Cancel; };
            this.buttonOk.Click += new EventHandler(buttonOk_Click);
        }

        //初始化控件
        private void InitControls(PublicEnums.EditType _eidtType, T_CONTENT content)
        {
            this.comboBoxState.Items.Add(CON_ALL);
            this.comboBoxState.BindComboBoxWithEnum(typeof(PublicEnums.State));
            comboBoxInterverDays.SelectedIndex = 0;

            if (_eidtType == PublicEnums.EditType.New)
            {
                this.comboBoxState.Text = PublicEnums.State.未完成.ToString();
                labelFinishTime.Visible = false;
                dateTimePickerFinish.Visible = false;
                this.dateTimePickerPredictFinishTime.Value = DateTime.Now.AddDays(1);
            }
            else if (_eidtType == PublicEnums.EditType.Mondify)
            {
                labelFinishTime.Visible = false;
                dateTimePickerFinish.Visible = false;
            }
            else//view
            {
                this.textBoxTitle.Enabled = false;
                this.buttonOk.Visible = false;
                this.buttonCancel.Text = "关闭(&C)";
                dateTimePickerPredictFinishTime.Enabled = false;
                comboBoxState.Enabled = false;
                dateTimePickerFinish.Enabled = false;
                dateTimePickerFinish.Visible = true;
                labelFinishTime.Visible = true;
                textBoxContent.Enabled = false;
                dateTimePickerNotifyTime.Enabled = false;
                comboBoxInterverDays.Enabled = false;
                checkBoxExe.Enabled = false;
                buttonExe.Enabled = false;
                if (_content.State != PublicEnums.State.完成.ToString())
                {
                    labelFinishTime.Visible = false;
                    dateTimePickerFinish.Visible = false;
                }
            }
        }

        //给控件赋值
        private void SetControlsValue(T_CONTENT content)
        {
            this.textBoxTitle.Text = content.ContentTitle;
            this.dateTimePickerPredictFinishTime.Value = content.Time_Predict_Finish;
            this.comboBoxState.Text = content.State;
            if (content.State == PublicEnums.State.完成.ToString())
            {
                this.dateTimePickerFinish.Value = content.Time_Finish;
            }
            textBoxContent.Text = content.ContentText;

            DateTime time = new DateTime(1900, 12, 31, content.Time_Notify / 60, content.Time_Notify % 60, 0, 0);
            this.dateTimePickerNotifyTime.Value = time;

            this.comboBoxInterverDays.Text = content.Interval_Days.ToString();
            this.checkBoxExe.Checked = content.Is_Exe_Notify == 1 ? true : false;
            this.textBoxExePath.Text = content.Exe_Path;
        }

        //点击确定按钮
        private void buttonOk_Click(object sender, EventArgs e)
        {
            if (this.Check())
            {
                _content.ContentText = this.textBoxContent.Text.Trim();
                _content.ContentTitle = this.textBoxTitle.Text.Trim();
                _content.Exe_Path = this.textBoxExePath.Text;
                _content.Interval_Days = Convert.ToInt32(this.comboBoxInterverDays.Text.Trim());
                _content.Is_Exe_Notify = this.checkBoxExe.Checked ? 1 : 0;
                if (_eidtType == PublicEnums.EditType.New)
                {
                    _content.Time_Create = DateTime.Now;
                }
                _content.Time_Notify = this.dateTimePickerNotifyTime.Value.Hour * 60 + this.dateTimePickerNotifyTime.Value.Minute;
                _content.Time_Predict_Finish = this.dateTimePickerPredictFinishTime.Value;

                this.DialogResult = System.Windows.Forms.DialogResult.OK;
            }
        }

        //检查合法性
        private bool Check()
        {
            if (this.textBoxTitle.Text.Trim() == string.Empty)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("标题不能为空！");
                this.textBoxTitle.Focus();
                return false;
            }
            using (TodoListEntities entities = new TodoListEntities())
            {
                if (_eidtType == PublicEnums.EditType.New && true == entities.T_CONTENT.Any(c => c.ContentTitle == this.textBoxTitle.Text.Trim() && c.Catalog_Id == _catalogId))
                {
                    PublicUnit.ControlsHelper.ShowInforMessage("相同的标题已经存在！");
                    return false;
                }
                else if (_eidtType == PublicEnums.EditType.Mondify && true == entities.T_CONTENT.Any(c => c.ContentTitle == this.textBoxTitle.Text.Trim() && c.Catalog_Id == _catalogId && c.Content_Id != _content.Content_Id) )
                {
                    PublicUnit.ControlsHelper.ShowInforMessage("相同的标题已经存在!");
                    return false;
                }
            }
            if (dateTimePickerPredictFinishTime.Value <= DateTime.Now)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("预计完成时间必须大于当前时间!");
                this.dateTimePickerPredictFinishTime.Focus();
                return false;
            }
            if (comboBoxInterverDays.Text.Trim() == string.Empty)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("提醒间隔必须填写!");
                this.comboBoxInterverDays.Focus();
                return false;
            }
            int temp;
            if (false == Int32.TryParse(this.comboBoxInterverDays.Text.Trim(),out temp))
            {
                PublicUnit.ControlsHelper.ShowInforMessage("提醒间隔必须输入大于0且小于等于" + Int32.MaxValue.ToString() +"的整数!");
                this.comboBoxInterverDays.Focus();
                return false;
            }
            if (Convert.ToInt32(this.comboBoxInterverDays.Text.Trim()) <= 0)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("提醒间隔必须输入大于0且小于等于" + Int32.MaxValue.ToString() + "的整数!");
                this.comboBoxInterverDays.Focus();
                return false;
            }
            if (this.checkBoxExe.Checked && this.textBoxExePath.Text == string.Empty)
            {
                PublicUnit.ControlsHelper.ShowInforMessage("请选择要执行的程序！");
                this.buttonExe.Focus();
                return false;
            }
            return true;
        }

        //选择可执行文件
        private void buttonExe_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                this.textBoxExePath.Text = dialog.FileName;
            }
        }

        #region 公共方法或属性

        /// <summary>
        /// 打开编辑窗体，如果是新建，content为空
        /// </summary>
        /// <param name="content">编辑的对象</param>
        /// <returns>成功：OK；取消：Cancel</returns>
        public DialogResult ShowDialog(T_CONTENT content,int catalogId,TodoList.PublicUnit.PublicEnums.EditType type)
        {
            _catalogId = catalogId;
            _eidtType = type;
            if (content == null)
            {
                _content = new T_CONTENT();
            }
            else
            {
                _content = content;
                this.SetControlsValue(_content);
            }
            this.InitControls(_eidtType,content);
            return this.ShowDialog();
        }

        /// <summary>
        /// 获取T_Content对象
        /// </summary>
        public T_CONTENT Content
        {
            get
            {
                return _content;
            }
        }

        #endregion
    }
}
