﻿/*
 * author:Joey Zhao
 * date:2010-10-3
 * descirbe：main form
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using TodoList.Bussiness;

namespace TodoList
{
    public partial class FormMain : Form
    {
        private FormCatalog _catalog;//目录
        private FormList _list;//todo列表
        private FormStatistics _statistics;//统计
        private FormNote _note;//便笺
        private bool _closeSign;

        public FormMain()
        {
            InitializeComponent();
            this.ToolStripMenuItemNote.Click += new EventHandler(this.ToolStripMenuItemNote_Click);
            this.SToolStripMenuItemStatistics.Click += new EventHandler(this.ToolStripMenuItemStatistics_Click);
            this.ToolStripMenuItemTodo.Click += new EventHandler(this.ToolStripMenuItemTodo_Click);

            this.notifyIcon1.DoubleClick += new EventHandler(notifyIcon1_DoubleClick);
            this.FormClosing += new FormClosingEventHandler(FormMain_FormClosing);
            this.Load += new EventHandler(FormMain_Load);
            this.ToolStripMenuItemClose.Click += new EventHandler(ToolStripMenuItemClose_Click);
            this.ToolStripMenuItemNotNotify.Click += new EventHandler(ToolStripMenuItemNotNotify_Click);

            this.dockPanel1.ActiveContentChanged += new EventHandler(dockPanel1_ActiveContentChanged);
        }

        //当前获得窗体发生改变时触发
        private void dockPanel1_ActiveContentChanged(object sender, EventArgs e)
        {
            if (this.dockPanel1.ActiveContent == _statistics || this.dockPanel1.ActiveContent == _note)
            {
                _catalog.CatalogEnabled = false;
            }
            else if (this.dockPanel1.ActiveContent == _list)
            {
                _catalog.CatalogEnabled = true;
            }
        }

        //点击菜单“今日未提醒任务”
        private void ToolStripMenuItemNotNotify_Click(object sender, EventArgs e)
        {
            FormNotNotifyTask form = new FormNotNotifyTask();
            int x = Screen.PrimaryScreen.WorkingArea.Width - form.Width - 15;
            int y = Screen.PrimaryScreen.WorkingArea.Height - form.Height;
            form.Location = new System.Drawing.Point(x, y);
            form.TopMost = true;
            form.Show();
        }

        //点击退出菜单
        private void ToolStripMenuItemClose_Click(object sender, EventArgs e)
        {
            if (PublicUnit.ControlsHelper.ShowOkCancelMessage("确实要退出吗？","退出确认"))
            {
                _closeSign = true;
                this.Close();
            }
        }

        //Load事件
        private void FormMain_Load(object sender, EventArgs e)
        {
            new ScanTimeOutDate().BeingScan();
             _list = new FormList();
            _catalog = new FormCatalog();
            _statistics = new FormStatistics();
            _note = new FormNote();
            _catalog.CurrentCatalogChanged += new EventHandler<CatalogEventArgs>(this.Catalog_CurrentCatalogChanged);
            _catalog.Show(this.dockPanel1, DockState.DockLeft);
            _list.Show(this.dockPanel1);
            NotifyList.GetInstance().StartScan();//开始扫描需要提醒的数据
        }

        //当前目录发生改变时事件
        private void Catalog_CurrentCatalogChanged(object sender, CatalogEventArgs e)
        {
            _list.CatalogId = e.CatalogId;
            _list.BindData();//当前目录改变后重新加载数据
        }

        //点击关闭窗体按钮
        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_closeSign == false)
            {
                e.Cancel = true;
                this.WindowState = FormWindowState.Minimized;
                this.Visible = false;
            }
        }

        //双击托盘图片
        private void notifyIcon1_DoubleClick(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.Visible = true;
                this.WindowState = FormWindowState.Normal;
            }
            else if (this.WindowState == FormWindowState.Normal)
            {
                this.Activate();
            }
        }

        //点击“事宜”菜单
        private void ToolStripMenuItemTodo_Click(object sender, EventArgs e)
        {
            if (!this.dockPanel1.Contents.Contains(_list))
            {
                _list = new FormList();
                _list.CatalogId = _catalog.CurrentCatalogId;
            }
            this.ShowDockContent(_list);
        }

        //点击“统计”菜单
        private void ToolStripMenuItemStatistics_Click(object sender, EventArgs e)
        {
            if (!this.dockPanel1.Contents.Contains(_statistics))
            {
                _statistics = new FormStatistics();
            }
            this.ShowDockContent(_statistics);
        }

        //点击“便笺”菜单
        private void ToolStripMenuItemNote_Click(object sender, EventArgs e)
        {
            if (!this.dockPanel1.Contents.Contains(_note))
            {
                _note = new FormNote();
            }
            this.ShowDockContent(_note);
        }

        //显示参数给定的子窗体
        private void ShowDockContent(DockContent content)
        {
            if (content.Visible == false)
            {
                content.Show(this.dockPanel1);
            }
            else if (content.IsActivated == false)
            {
                content.Activate();
            }
        }
    }
}
